################################################################################################################
# Script for generation of IFPs from dissociation trajectories generated by Gromac (RAMD procedure)
# for usage, please adjust directory/file structure accordingly!
################################################################################################################

#"G-Protein Coupled Receptor-Ligand Dissociation Rates and Mechanisms from tauRAMD Simulations" 
# by Daria B. Kokh, Rebecca C. Wade
# submitte to JCTC , June 2021

################################################################################################################

# Packages required:
# numpy ( 1.18.1)
# pandas (1.0.2)
# MDAnalysis (0.20.1)
# rdkit (2019.09.3)
# code is written on Python 3.x and tested on the version 3.7

################################################################################################################

#02.06.2021
#Copyright (c) 2020
#Released under the EUPL Licence, v1.2 or any higher version

################################################################################################################

#Author: Daria Kokh
#Daria.Kokh@h-its.org or mcmsoft@h-its.org
#Heidelberg Institute of Theoretical Studies (HITS, www.h-its.org)
#Schloss-Wolfsbrunnenweg 35
#69118 Heidelberg, Germany

################################################################################################################

import glob, os
import sys
import subprocess
import numpy as np

import pandas as pd
from pandas import ExcelFile 

from matplotlib import *
from matplotlib import cm
import matplotlib.ticker
import  pylab as plt

from scipy import stats

from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import EState
from rdkit.Chem import MolDb
from rdkit.Chem import QED
from rdkit.Chem import rdchem
from rdkit.Chem import ChemicalFeatures
from rdkit import RDConfig

import MDAnalysis as mda
from MDAnalysis.lib.formats.libdcd import DCDFile
from MDAnalysis.analysis import contacts,align,rms
from MDAnalysis.analysis.base import AnalysisFromFunction
from MDAnalysis.coordinates.memory import MemoryReader

from sklearn import linear_model
from sklearn import preprocessing

from rdkit import Chem

from Scripts.IFP_generation import *
from Scripts.Trajectories import *

repl_number = 1
ligand = ""
DIR_all = os. getcwd()
DIR_namd = "/GROMACS-2020.2/Replica*"
DIR_ramd = "/RAMD-2020.2/"
ref = "b2AR-PDB.pdb"

ramd_trj = "/TRSL1-*/*whole.xtc"
ligandPDB = "alprenolol.pdb" 
ligandMol2 = "alprenolol.mol2"
ramd_trj_org = "traj_comp.xtc"

IFP_output_name = "b2AR-unwtapped-WB-SL-700-RAMD_1"

tr_F = []
tr_F_name = []

#------------ loop over ligands (in fact, for b2AR only one ligand)
#------------- extract IFPs from trajectories 

if os.path.exists(DIR_all):
    dir_ligand = DIR_all
    i =0
    if  os.path.isfile(dir_ligand+"/"+ref):
            print(dir_ligand)
            sys.stdout.flush()
            lig_name=os.path.basename(dir_ligand)
            print("=======================",lig_name,"===============================")        
            tr = trajectories(dir_ligand,namd_tmpl= DIR_namd, ramd_tmpl= DIR_ramd,ligand_pdb=ligandPDB,\
                          ligand_mol2=ligandMol2,namd_traj_tmpl = "*xtc",ramd_traj_tmpl = ramd_trj,pdb=ref)
            if(len(tr.ramd.repl_traj) == 0):
                print("\nSKIP ",i,dir_ligand,"RAMD data are missing; NAMD trajectories generated",len(tr.namd.repl_traj))
            else:
                print("\n",i,tr.tau_exp,"NAMD generated %s; RAMD generated %s  " %(len(tr.namd.repl_traj),len(tr.ramd.repl_traj)))
                tr.ramd.scan_ramd()
            tr_F.append(tr)
            tr_F_name.append(os.path.basename(dir_ligand))
            sys.stdout.flush()
    else: print(dir_ligand+"/"+ref+" was not found")

#------------- save pkl DataFrames with IFP and some additional parameters
step = 1
for tr,name in zip(tr_F,tr_F_name):
   print("==================  ",name,"  =======================")
   auxi_selection = ["((resid 88:100) and (not type H))","((resid 122:126) and (not type H))"] # these are additional parts for the protein, whose  RMSD values will be stored in IFP matrix
   tr.analysis_all_ramd(WB_analysis = True, Lipids = ["CHL","OL","PC","PE","PA","PH-"],auxi_selection = auxi_selection,step_analysis=step, start_analysis=-700)
    # save IFP
   IFP_table = tr.ramd.IFP_save(DIR_all+IFP_output_name+repl_number+".pkl")
   sys.stdout.flush()
   for repl_name in tr.ramd.repl_traj:
      for traj_name in repl_name:
           dir_current = os.path.dirname(traj_name)
           u_org =  mda.Universe(DIR_all+ligand+"/"+ref,dir_current+"/"+ramd_trj_org)
           print("Complete TRJ ",traj_name, len(u_org.trajectory))



